#include <locale.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "geodesie.h"
#include "misc.h"
#include "raf98.h"

#include "convert.dll.h"

#define WGS84 0
#define NTF   1

#define CARTESIENNES  0
#define GEOGRAPHIQUES 1
#define PLANES        2

int convert(const char *inputFilename, const char *outputFilename, const char *outputProjection, double ha) {
   data in={0}, out={0};

   // --- Constants Initialization ----------------------------------------------
   in.sysg=WGS84;
      in.cstes.a=6378137.0;
      in.cstes.f=1/298.257223563;
      in.cstes.e=E(in.cstes.a, in.cstes.a*(1.0-in.cstes.f)); // e=f(a, b) w/ b=a(1-f)
   in.proj=0; // none
   in.coord=GEOGRAPHIQUES;

   out.sysg=NTF;
      out.cstes.a=6378249.2;
      out.cstes.b=6356515.0;
      out.cstes.e=E(out.cstes.a, out.cstes.b);
      out.cstes.lambda0=deg2rad(2.337229167); // 0 grades Paris (= 2 20' 14,025" E Greenwich)
      out.cstes.T.x=-168.0;
      out.cstes.T.y=-60.0;
      out.cstes.T.z=320.0;
      out.cstes.D=0.0;
      out.cstes.R.x=0.0;
      out.cstes.R.y=0.0;
      out.cstes.R.z=0.0;

   if(strcmp(outputProjection, "Lb1") == 0) {
      out.proj=1;

      out.cstes.n=0.7604059656;
      out.cstes.C=11603796.98;
      out.cstes.Xs=600000.0;
      out.cstes.Ys=5657616.674;
   } else if(strcmp(outputProjection, "Lb2") == 0) {
      out.proj=2;

      out.cstes.n=0.7289686274;
      out.cstes.C=11745793.39;
      out.cstes.Xs=600000.0;
      out.cstes.Ys=6199695.768;
   } else if(strcmp(outputProjection, "Lb2e") == 0) {
      out.proj=3;

      out.cstes.n=0.7289686274;
      out.cstes.C=11745793.39;
      out.cstes.Xs=600000.0;
      out.cstes.Ys=8199695.768;
   } else if(strcmp(outputProjection, "Lb3") == 0) {
      out.proj=4;

      out.cstes.n=0.6959127966;
      out.cstes.C=11947992.52;
      out.cstes.Xs=600000.0;
      out.cstes.Ys=6791905.085;
   } else if(strcmp(outputProjection, "Lb4") == 0) {
      out.proj=5;

      out.cstes.n=0.6712679322;
      out.cstes.C=12136281.99;
      out.cstes.Xs=234.358;
      out.cstes.Ys=7239161.542;
   }

   out.coord=PLANES;

   // --- Files Initialization --------------------------------------------------
   FILE *pInputFile=NULL, *pOutputFile=NULL;

   pOutputFile=fopen(outputFilename, "wt");

   if(pOutputFile != NULL) {
      fprintf(pOutputFile, "# Format: North East ReducedDepth\n");

      pInputFile=fopen(inputFilename, "rt");

      if(pInputFile != NULL) {
         int nb=1; // Line number wrote in outputFile

         printf("RAF98 grid loading...");
         int raf98loaded=loadRAF98("raf98.gra");
         printf(" ok\n");

         if(raf98loaded) {
            char fileLine[1024]=""; // same as in Notepad

            printf("Latitude   Longitude  Depth Nord               Est               Reduced Depth\n");

            while(fgets(fileLine, 1024, pInputFile)) {
               if(fileLine[0] != '#') { // symbol of comment met
                  double lat, lon, depth, he;

                  sscanf(fileLine, "%*s %lf %lf %lf %lf %*s %*s\n", &lat, &lon, &depth, &he);

                  // Read input coordinates
                  coord_geographiques src;
                     src.phiv=deg2rad(lat);
                     src.lambda=deg2rad(lon);
                     src.he=he;

                  coord_cartesiennes resTemp=similitude3D_7params(cg2cc(src, in.cstes.a, in.cstes.e), out.cstes.T, out.cstes.D, out.cstes.R);

                  // Write output coordinates by type
                  coord_planes res=cg2cp(cc2cg(resTemp, out.cstes.a, out.cstes.e), out.cstes.C, out.cstes.e, out.cstes.lambda0, out.cstes.n, out.cstes.Xs, out.cstes.Ys);

                  double alt=altitude(lat, lon, depth, he, ha);

                  printf("%-10.7lf %-10.7lf %-5.2f %-18.10lf %-17.10lf %-10.7lf\n", lat, lon, depth, res.N, res.E, alt);

                  fprintf(pOutputFile, "%.3lf %.3lf %.2lf\n", res.N, res.E, alt);

                  nb++;
               } // else: line is a comment.
            }
         } else {
            printf("MIS-convert: [ERROR] Loading `raf98.gra' failed.\n");
         }

         fclose(pInputFile);

         printf("Finished.\n");
         printf("Output file: %s [%d line%s\n", outputFilename, nb, ((nb > 1) ? "s]" : "]"));
      } else {
         printf("MIS-convert: [ERROR] File not found. [%s]\n", inputFilename);
      }

      fclose(pOutputFile);
   } else {
      printf("MIS-convert: [ERROR] Output file can not be appended. [%s]\n", outputFilename);
   }

   return 0;
}
