#include <math.h>

#include "misc.h"

#include "geodesie.h"

double E(double a, double b) {
/* in: a, b
  out: E
*/
   return sqrt((sq(a)-sq(b))/sq(a));
}

double L(double phiv, double e) {
/* in: phiv, e
  out: L
*/
   return log(tan(pi/4.0+phiv/2.0)*pow((1.0-e*sin(phiv))/(1.0+e*sin(phiv)), e/2.0));
// return 0.5*log((1.0+sin(cg.phiv))/(1.0-sin(cg.phiv)))-((e/2.0)*log((1.0+e*sin(cg.phiv))/(1.0-e*sin(cg.phiv))));
}

double N(double phiv, double a, double e) {
/* in: phiv, a, e
  out: N
*/
   return a/sqrt(1.0-sq(e)*sq(sin(phiv)));
}

coord_geographiques cc2cg(coord_cartesiennes cc, double a, double e) {
/* in: (X, Y, Z), a, e, epsi
  out: phiv, lambda, he
*/
   coord_geographiques result;

/* source: transfo.pdf */

   result.lambda=atan(cc.Y/cc.X);

   double f=1.0-sqrt(1.0-sq(e));
   double mu=atan((cc.Z/sqrt(sq(cc.X)+sq(cc.Y)))*((1.0-f)+((sq(e)*a)/sqrt(sq(cc.X)+sq(cc.Y)+sq(cc.Z)))));

   result.phiv=atan((cc.Z*(1.0-f)+sq(e)*a*cube(sin(mu)))/((1.0-f)*(sqrt(sq(cc.X)+sq(cc.Y))-sq(e)*a*cube(cos(mu)))));
   result.he=(sqrt(sq(cc.X)+sq(cc.Y))*cos(result.phiv))+cc.Z*sin(result.phiv)-(a*sqrt(1.0-sq(e)*sq(sin(result.phiv))));

   return result;
}

coord_cartesiennes cg2cc(coord_geographiques cg, double a, double e) {
/* in: (phiv, lambda, he), a, e
  out: X, Y, Z
*/
   coord_cartesiennes result;

   double n=N(cg.phiv, a, e);

   result.X=(n+cg.he)*cos(cg.phiv)*cos(cg.lambda);
   result.Y=(n+cg.he)*cos(cg.phiv)*sin(cg.lambda);
   result.Z=(n*(1.0-sq(e))+cg.he)*sin(cg.phiv);

   return result;
}

coord_planes cg2cp(coord_geographiques cg, double C, double e, double lambda0, double n, double Xs, double Ys) {
/* in: (phiv, lambda), C, e, lambda0, n, Xs, Ys
  out: N, W
*/
   coord_planes result;

   double l=L(cg.phiv, e);
   double R=C*exp(-n*l);
   double gamma=n*(cg.lambda-lambda0);

   result.N=Ys-R*cos(gamma);
   result.E=Xs+R*sin(gamma);

   return result;
}

coord_cartesiennes similitude3D_7params(coord_cartesiennes cc, vector T, double D, vector R) {
/* in: U=(Ux, Uy, Uz), (Tx, Ty, Tz), D, (Rx, Ry, Rz)
  out: V=(Vx, Vy, Vz)
*/
   coord_cartesiennes result;

   double e=D+1.0;
   double delta=e*(sq(e)+sq(R.x)+sq(R.y)+sq(R.z));

   cc.X=cc.X-T.x;
   cc.Y=cc.Y-T.y;
   cc.Z=cc.Z-T.z;

   result.X=((sq(e)+sq(R.x))*cc.X+(e*R.z+R.x*R.y)*cc.Y+(R.x*R.z-e*R.y)*cc.Z)/delta;
   result.Y=((-e*R.z+R.x*R.y)*cc.X+(sq(e)+sq(R.y))*cc.Y+(e*R.x+R.y*R.z)*cc.Z)/delta;
   result.Z=((e*R.y+R.x*R.z)*cc.X+(-e*R.x+R.y*R.z)*cc.Y+(sq(e)+sq(R.z))*cc.Z)/delta;

   return result;
}
