#ifndef GEODESIE_H
#define GEODESIE_H

// --- Personalized types -------------------------------------------------------

// vector of translation or rotation
typedef struct {
   double x;
   double y;
   double z;
} vector;

// Cartesian co-ordinates
typedef struct {
   double X;
   double Y;
   double Z;
} coord_cartesiennes;

// Geographical co-ordinates
typedef struct {
   double phiv;   // latitude
   double lambda; // longitude
   double he;     // hauteur ellipsoidale
} coord_geographiques;

// Plane co-ordinates
typedef struct {
   double N; // latitude
   double E; // longitude
} coord_planes;

// --- Functions prototypes -----------------------------------------------------

// calculation of the first eccentricity
double E(double a, double b);

/* calculation of the isometric latitude on an ellipsoid of first eccentricity e
   at the point of latitude phi.
*/
double L(double phiv, double e);

// calculation the great normal
double N(double phiv, double a, double e);

/* transformation, for a given ellipsoid, of Cartesian co-ordinates of a point into
   ellipsoidal geographical co-ordinates by the method of Heiskanen-Moritz-Butcher.
*/
coord_geographiques cc2cg(coord_cartesiennes cc, double a, double e);

// transformation of ellipsoidal geographical co-ordinates into Cartesian co-ordinates
coord_cartesiennes cg2cc(coord_geographiques cg, double a, double e);

/* transformation of geographical co-ordinates into plane co-ordinates conforms
   to the conical projection of Lambert.
*/
coord_planes cg2cp(coord_geographiques cg, double C, double e, double lambda0, double n, double Xs, double Ys);

/* calculation of the three-dimensional Cartesian co-ordinates in the system (2)
   starting from a set of 7 parameters (3 translations, 1 scale factor and 3
   rotations) of passage of the system (1) towards the system (2), and of the
   three-dimensional Cartesian co-ordinates in the system (1).
*/
coord_cartesiennes similitude3D_7params(coord_cartesiennes cc, vector T, double D, vector R);

#endif
